/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/**
@fileoverview
This file is responsible for handling the mouse events.
The mouse listener runs all the time, don't add anything that will slow it down.
*/

/**
Allow Mozile to intercept mouse events. 
*/
function mozileAddMouseListener() {
document.addEventListener("mouseup", mozileMouse, false);
document.addEventListener("click", mozileMouseClick, false);
}
/**
Stop Mozile from intercepting mouse events.
*/
function mozileRemoveMouseListener() {
document.removeEventListener("mouseup", mozileMouse, false);
document.removeEventListener("click", mozileMouseClick, false);
}


/**
Mozile Mouse handler.
On every mouse click it checks to see if Mozile should be activated or deactivated.
Decides whether to autohide the toolbar, and when to activate the caret.
If within the browser ignore mouse click.
If click is on browser tab, determine if toolbar should be shown.
*/
function mozileMouse(event) {
	var t=event.target;
	try{
	if(t.ownerDocument.defaultView=="[object ChromeWindow]"){
		if(t.localName=="tabbrowser"){
			if(event.originalTarget.localName=="tab"){
				var tab=event.originalTarget;
				var b=event.target.getBrowserForTab(tab);
				if( "moziled" in b.contentDocument && b.contentDocument.moziled ){
					mozileTools.show();
				}else if(mozilePrefs.getBoolPref('mozile.autohideBool')) {
					mozileTools.hide();
				}
			}
		}
		//else this will catch mouse clicks in source edit textbox.
		return;
	}
	}catch(e){return;}

	if (mozileDetectOverride(t)) {
		return;
	}
	if(mozileDetect()) {
		var f = ["mozileMouse.js","mozileMouse"];
		mozile.bug.write(f,3,"Tag "+event.target.tagName);
		mozileDetectToolbar(); //Show toolbar and turn caret on.
	}	else {
		mozileNoDetectToolbar(t); //hide the toolbar (if autohide is on) and restore the caret state.
	}
}

function mozileDetectOverride( t ) {
	if (t.nodeType == 1) {
		if ( (t instanceof HTMLInputElement ) ||
			   (t instanceof HTMLTextAreaElement ) ) {
			t.focus();
			mozileNoDetect();
			mozileNoDetectToolbar(t);
			return true;
		}
	}
	return false;
}

/**
Show the toolbar and caret.
*/
function mozileDetectToolbar() {
	if(mozileTools.toolbar.collapsed) {
		mozileTools.show();
	}
	mozilePrefs.setBoolPref('accessibility.browsewithcaret', true);
}
/**
Show or hide the toolbar and caret as required.
*/
function mozileNoDetectToolbar( target ) {
	if( "moziled" in target.ownerDocument && target.ownerDocument.moziled ) {
		if(mozileTools.toolbar.collapsed){
			mozileTools.hide();
		}
	} else if( !mozileTools.toolbar.collapsed && mozilePrefs.getBoolPref('mozile.autohideBool')) {
		mozileTools.hide();
	}
	mozileRestoreCaret();
}

function mozileRestoreCaret() {
	mozilePrefs.setBoolPref('accessibility.browsewithcaret', mozilePrefs.getBoolPref('mozile.caretAlwaysOnBool'));
}

/**
Mouse click event handler - checks for an HTML link.
Only looks at "left" mouse button click.
If the browser override preference is set the click away will be prevented.

Mouse click occurs after mouse up, so mozileDetect will ALWAYS have been just called.
i.e. MozileTarget will be set if the click is in an editable area.
However, if the first click in the editable area is on a link, mouseup won't detect it.
This is because the point clicked does not get selected, I think.
Hence, if the document is Mozile aware, we also have to check if the anchor element is
editable.

*/
function mozileMouseClick(event) {
	if(event.button!==0) {
		return;
	}
	if(!mozilePrefs.getBoolPref("mozile.acceleratorBool")) {
		return;
	}
	if(mozileTarget || ("moziled" in event.target.ownerDocument)) {
		mozileLinkDetect(event);
	}
}

function mozileLinkDetect(event) {
	var t = event.target;
	var docEl = t.ownerDocument.documentElement;
	do {
		if (t.nodeType==1) {
			if (t instanceof HTMLAnchorElement) {
				if ( t.ownerDocument.defaultView.getComputedStyle(t, null).MozUserModify=="read-write" ) {
					event.preventDefault();
					document.commandDispatcher.focusedWindow.getSelection().collapse(event.rangeParent, event.rangeOffset);
					mozileDetect(); //Force update for new selection.
					return;
				}
			}
		}
		t = t.parentNode; 
	} while ( t && t!=docEl );
}