/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code (www.playsophy.com).
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * - Christian Stocker (Bitflux)
 *
 * ***** END LICENSE BLOCK ***** */

/*
 * Global Mozilla singleton that provides access native objects needed in Mozile:
 * - clipboard
 *
 * Access using "mozilla.<methodName>" ex/ "mozilla.getClipboard()"
 *
 */

/*******************************************************************************************************
 * Global object - ideally set up at browser initialization but is only a JS object here.
 *******************************************************************************************************/

var mozilla = new Moz();

function Moz()
{
	this.__clipboard = null;
	try {
		// try enable xpconnect
		netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
	}
	catch(e) // POST05: should really check exception type and not just assume!
	{
		this.__allowedNativeCalls = false;
		return;
	}
	this.__allowedNativeCalls = true;
}

/**
 * May get the Mozilla clipboard if available - otherwise get IFrame clipboard
 */
Moz.prototype.getClipboard = function() 
{
	if(!this.__clipboard) 
	{
		if(this.__allowedNativeCalls)
			this.__clipboard = new MozClipboard();
		else
			this.__clipboard = new NonNativeClipboard();
	}

	return this.__clipboard;
}

/*******************************************************************************************************
 * Non native clipboard: used if native support is not available - only handles
 *
 * POST05: 
 * - Christian's IFrame clipboard: enhance nonNativeClipboard to use an id'ed IFrame to paste too and from
 * a Mozile page from other applications.
 *******************************************************************************************************/

function NonNativeClipboard()
{
	this.__copiedData = null;
}

NonNativeClipboard.prototype.getData = function(dataFlavor)
{
	// return copy of internal contents if any
	if(this.__copiedData)
	{
		return this.__copiedData;
	}
	return undefined;
}

NonNativeClipboard.prototype.setData = function(data, dataFlavor)
{
	this.__copiedData = data;
}

/*******************************************************************************************************
 * Moz clipboard
 *
 * Access with call "mozilla.getClipboard()" [ala window.getSelection()]
 * 
 * Two references:
 * - http://www.mozilla.org/xpfe/xptoolkit/introClipDD.html
 * - http://www.xulplanet.com/tutorials/xultu/clipboard.html
 * - http://www.xulplanet.com/references/xpcomref/nsITransferable.html
 * - http://lxr.mozilla.org/seamonkey/source/widget/public/nsIClipboard.idl
 * - http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnwue/html/ch06f.asp
 *
 * Note: normally would be a method of "window" but window may be created before the Mozile scripts are loaded. 
 *
 * POST05: 
 * - check out gClipboardHelper
 * - may need to return a tuple of {data, dataFlavor} - or take the "downgrade to text" approach.
 * - overlap with domlevel3 in transforming data types etc ... may layer standard implementations over these
 * - copy html - try to get back as text - it fails. Change my impl to downgrade to text ie/ get all as text!
 * - how to get XML as XML (doesn't seem to work!); other formats? Word? [after all Moz's clipboard is a nice
 * portable i/f for clipboard]
 * - http://lxr.mozilla.org/seamonkey/source/xpfe/global/resources/content/nsTransferable.js#301 (seem
 * to be moz specific data flavors - add to list? [found this file late - consider stuff in it for
 * inclusion here]
 * - marry this with support for drag and drop of elements into and out of editable area
 * - ultimately consts for i/fs and services (const nsISupports = Components.interfaces.nsISupports; with
 * proper object wrappers for all the things I need (don't try domlevel3 directly over native)
 * 
 ******************************************************************************************************/

function MozClipboard()
{
	// enable xpconnect
	netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');

	// create an object for accessing the native clipboard
	this.clip = Components.classes['@mozilla.org/widget/clipboard;1'].createInstance(Components.interfaces.nsIClipboard);
}

// POST05: redo as consts (scale's better)
MozClipboard.TEXT_FLAVOR = "text/unicode";
MozClipboard.HTML_FLAVOR = "text/html";

/*
 * Returns content of clipboard
 * 
 * - return requested flavor if available; otherwise return text if available; other return null
 *
 * POST05:
 * - do data downgrading too ie/ if text is not available but text is requested then return another format AS text!
 * (required as if I copy html and then try to get it as text, I get nothing now!)
 */
MozClipboard.prototype.getData = function(dataFlavor)
{
	// enable xpconnect: must do again in this function even though enabled for constructor (seems daft)
	netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');

	var trans = Components.classes['@mozilla.org/widget/transferable;1'].createInstance(Components.interfaces.nsITransferable);
	if(dataFlavor != MozClipboard.TEXT_FLAVOR)
		trans.addDataFlavor(dataFlavor); // can't share trans: need to do required flavor first or never shows up!
	trans.addDataFlavor(MozClipboard.TEXT_FLAVOR);

	this.clip.getData(trans, this.clip.kGlobalClipboard); // must use global - selection clipboard doesn't work and I don't know why!

	var str = new Object();
    	var len = new Object();

	try 
	{
		trans.getTransferData(dataFlavor, str, len);
	}
	catch(e)
	{
		try 
		{
			trans.getTransferData(MozClipboard.TEXT_FLAVOR, str, len);
		}
		catch(e)
		{
			return null; // nothing we can copy
		}
	}

	str= str.value.QueryInterface(Components.interfaces.nsISupportsString);
	if(str)
		str = (str.data.substring(0,len.value / 2));
	return str;
}

/**
 * Add data to the clipboard: data in the form of a string
 */
MozClipboard.prototype.setData = function(data, dataFlavor)
{
	netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');

	// xpcom wrapper for data - note: wstring doesn't seem to work so choose string - seems arbitrary but it works!
	var str = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
	str.data = data;

	var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
	trans.addDataFlavor(dataFlavor);
	trans.setTransferData(dataFlavor, str, data.length*2); 

	this.clip.setData(trans, null, this.clip.kGlobalClipboard); // note: I don't know the difference between global and selection but selection works!

	if ("forceDataToClipboard" in this.clip) //Not present in all platforms, so need to test for it.
		this.clip.forceDataToClipboard(this.clip.kGlobalClipboard);
}
